<?php
/**
 * A berkeley db based cache for free/busy data.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * A berkeley db based cache for free/busy data.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache/DB.php,v 1.1.2.1 2010-05-16 06:22:09 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_DB {

    /**
     * The directory that should be used for caching.
     *
     * @var string
     */
    var $_cache_dir;

    /**
     * The resource handle into the database.
     *
     * @var resource
     */
    var $_db = false;

    /**
     * The format of the database.
     *
     * @var string
     */
    var $_dbformat;

    /**
     * The type of this cache.
     *
     * @var string
     */
    var $_type = '';

    /**
     * The directory that should be used for caching.
     *
     * @var string
     */
    function Horde_Kolab_FreeBusy_Cache_DB($cache_dir) {
        global $conf;

        $this->_cache_dir = $cache_dir;

        if (!empty($conf['fb']['dbformat'])) {
            $this->_dbformat = $conf['fb']['dbformat'];
        } else {
            $this->_dbformat = 'db4';
        }

        /* make sure that a database really exists before accessing it */
        if (!file_exists($this->_cache_dir . '/' . $this->_type . 'cache.db')) {
            $result = $this->_open();
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
            $this->_close();
        }

    }

    /**
     * Open the database.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function _open()
    {
        if ($this->_db !== false) {
            return true;
        }

        $dbfile = $this->_cache_dir . '/' . $this->_type . 'cache.db';
        $this->_db = dba_open($dbfile, 'cd', $this->_dbformat);
        if ($this->_db === false) {
            return PEAR::raiseError(sprintf("Unable to open freebusy cache db %s", $dbfile));
        }
        return true;
    }

    /**
     * Close the database.
     */
    function _close()
    {
        if ($this->_db !== false) {
            dba_close($this->_db);
        }
        $this->_db = false;
    }

    /**
     * Set a cache file as irrelevant for a user.
     *
     * @param string $filename The cache file to remove.
     * @param string $uid      The user ID.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function _remove($filename, $uid)
    {
        $result = $this->_open();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        if (dba_exists($uid, $this->_db)) {
            $lst = dba_fetch($uid, $this->_db);
            $lst = explode(',', $lst);
            $lst = array_diff($lst, array($filename));
            $result = dba_replace($uid, join(',', $lst), $this->_db);
            if ($result === false) {
                $result = PEAR::raiseError(sprintf("Unable to set db value for uid %s", $uid));
            }
        }
        $this->_close();

        return $result;
    }

    /**
     * Set a cache file as relevant for a user.
     *
     * @param string $filename The cache file to add.
     * @param string $uid      The user ID.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function _add($filename, $uid)
    {
        if (empty($filename)) {
            return true;
        }

        $result = $this->_open();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        if (dba_exists($uid, $this->_db)) {
            $lst = dba_fetch($uid, $this->_db);
            $lst = explode(',', $lst);
            $lst[] = $filename;
            $result = dba_replace($uid, join(',', array_keys(array_flip($lst))), $this->_db);
            if ($result === false) {
                $result = PEAR::raiseError(sprintf("Unable to set db value for uid %s", $uid));
            }
        } else {
            $result = dba_insert($uid, $filename, $this->_db);
            if ($result === false) {
                $result = PEAR::raiseError(sprintf("Unable to set db value for uid %s", $uid));
            }
        }
        $this->_close();

        return $result;
    }

    /**
     * Is the cache file relevant for the user?
     *
     * @param string $filename The cache file.
     * @param string $uid      The user ID.
     *
     * @return boolean|PEAR_Error True if the cache file is relevant.
     */
    function has($filename, $uid)
    {
        $result = $this->_open();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = false;
        if (dba_exists($uid, $this->_db)) {
            $lst = dba_fetch($uid, $this->_db);
            $lst = explode(',', $lst);
            $result = in_array($filename, $lst);
        }
        $this->_close();

        return $result;
    }

    /**
     * Get the full list of relevant cache files for a uid.
     *
     * @param string $uid      The user ID.
     *
     * @return array|PEAR_Error The list of cache files.
     */
    function get($uid)
    {
        $result = $this->_open();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $result = array();
        if (dba_exists($uid, $this->_db)) {
            $lst = dba_fetch($uid, $this->_db);
            $lst = explode(',', $lst);
            $result = array_filter($lst, array($this, '_notEmpty'));
        }
        $this->_close();

        return $result;
    }

    /**
     * Check if the value is set.
     *
     * @param mixed $value  The value to check.
     *
     * @return boolean True if the value is set.
     */
    function _notEmpty($value)
    {
        return !empty($value);
    }

    /**
     * Attempts to return a reference to a concrete FreeBusyACLCache
     * instance. It will only create a new instance if no
     * FreeBusyACLCache instance currently exists.
     *
     * This method must be invoked as:
     *   <code>$var = &FreeBusyACLCache::singleton($cache_dir);</code>
     *
     * @static
     *
     * @param string $type       The type of the cache.
     * @param string $cache_dir  The directory for storing the cache.
     *
     * @return FreeBusyACLCache The concrete FreeBusyACLCache
     *                          reference, or false on an error.
     */
    function &singleton($type, $cache_dir)
    {
        static $cachedb = array();

        $signature = $type . $cache_dir;

        if (empty($cachedb[$signature])) {
            $class = 'Horde_Kolab_FreeBusy_Cache_DB_' . $type;
            $cachedb[$signature] = new $class($cache_dir);
        }

        return $cachedb[$signature];
    }
}
